'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.boundedChunk = exports.takeIdxAndSplit = exports.median = undefined;
exports.makeOwnerID = makeOwnerID;
exports.tagOwnerID = tagOwnerID;
exports.canMutate = canMutate;
exports.slice = slice;
exports.arrayClone = arrayClone;
exports.withoutIdx = withoutIdx;
exports.insert = insert;
exports.set = set;
exports.fastMap = fastMap;
exports.splitAt = splitAt;
exports.unshift = unshift;
exports.last = last;
exports.init = init;
exports.tail = tail;
exports.getEmptyIterator = getEmptyIterator;
exports.extend = extend;
exports.makeRef = makeRef;
exports.setRef = setRef;
exports.isSet = isSet;
exports.normalizeRangeSpec = normalizeRangeSpec;

var _constants = require('./constants');

var median = exports.median = function median(len) {
    return Math.ceil(len / 2);
};

function makeOwnerID() {
    return {};
}

function tagOwnerID(obj, ownerID) {
    obj.ownerID = ownerID;
    return obj;
}

function canMutate(obj, ownerID) {
    return ownerID && ownerID === obj.ownerID;
}

function allocateArray(ownerID, len) {
    return tagOwnerID(len ? new Array(len) : [], ownerID);
}

function slice(ownerID, start, end, arr) {
    var newLen = end - start;
    if (canMutate(arr, ownerID)) {
        var removeNFromStart = start;
        var removeNFromEnd = arr.length - end;
        while (removeNFromStart--) {
            arr.shift();
        }
        while (removeNFromEnd--) {
            arr.pop();
        }

        return arr;
    }

    var newArr = allocateArray(ownerID, newLen);
    for (var i = start; i < end; i++) {
        newArr[i - start] = arr[i];
    }
    return newArr;
}

function arrayClone(arr) {
    var len = arr.length;
    var copy = new Array(len);

    for (var i = 0; i < len; i++) {
        copy[i] = arr[i];
    }
    return copy;
}

function withoutIdx(ownerID, idx, arr) {
    var copied = canMutate(arr, ownerID) ? arr : tagOwnerID(arrayClone(arr), ownerID);

    copied.splice(idx, 1);
    return copied;
}

function insert(ownerID, idx, val, arr) {
    if (canMutate(arr, ownerID)) {
        arr.splice(idx, 0, val);
        return arr;
    }

    var newArrLen = arr.length + 1;
    var newArr = new Array(newArrLen);

    var i = 0;
    for (; i < idx; i++) {
        newArr[i] = arr[i];
    }

    newArr[i++] = val;

    for (; i < newArrLen; i++) {
        newArr[i] = arr[i - 1];
    }

    return newArr;
}

function set(ownerID, idx, val, arr) {
    var copied = canMutate(arr, ownerID) ? arr : tagOwnerID(arrayClone(arr), ownerID);

    copied[idx] = val;
    return copied;
}

function fastMap(fn, arr) {
    var copied = arrayClone(arr);
    var len = arr.length;
    for (var i = 0; i < len; i++) {
        copied[i] = fn(arr[i], i);
    }
    return copied;
}

function splitAt(ownerID, idx, arr) {
    var arrLen = arr.length;
    var firstArrLen = idx;
    var secondArrLen = arrLen - idx;
    var secondArr = allocateArray(ownerID, secondArrLen);

    var firstArr = allocateArray(ownerID, firstArrLen);

    for (var i = 0; i < idx; i++) {
        firstArr[i] = arr[i];
    }

    for (var _i = idx; _i < arrLen; _i++) {
        secondArr[_i - idx] = arr[_i];
    }

    return [firstArr, secondArr];
}

function unshift(ownerID, value, arr) {
    return insert(ownerID, 0, value, arr);
}

var takeIdxAndSplit = exports.takeIdxAndSplit = function takeIdxAndSplit(ownerID, idx, arr) {
    var cutoff = idx;
    var a1len = cutoff;
    var arr1 = slice(ownerID, 0, a1len, arr);
    var arr2 = slice(ownerID, cutoff + 1, arr.length, arr);
    return [arr1, arr[cutoff], arr2];
};

function last(arr) {
    return arr[arr.length - 1];
}

function init(ownerID, arr) {
    if (canMutate(arr, ownerID)) {
        if (arr.length === 0) return arr;
        arr.pop();
        return arr;
    }

    if (arr.length <= 1) return allocateArray(ownerID);
    return slice(ownerID, 0, arr.length - 1, arr);
}

function tail(ownerID, arr) {
    if (canMutate(arr, ownerID)) {
        arr.shift();
        return arr;
    }

    if (arr.length <= 1) return tagOwnerID([], ownerID);
    return slice(ownerID, 1, arr.length, arr);
}

function getEmptyIterator() {
    var iterator = {
        next: function next() {
            return { done: true };
        }
    };

    iterator[_constants.ITERATOR_PROPNAME] = function () {
        return iterator;
    };

    return iterator;
}

var boundedChunk = exports.boundedChunk = function boundedChunk(min, max, arr) {
    var arrLen = arr.length;
    if (!arr.length) return [];
    if (arr.length <= max) {
        return [arr];
    }

    var avg = Math.ceil((min + max) / 2);
    var parts = arrLen / avg;
    var chunkCount = Math.ceil(parts);
    var splitsize = 1 / chunkCount * arrLen;
    var chunks = new Array(chunkCount);
    for (var i = 0; i < chunkCount; i++) {
        chunks[i] = arr.slice(Math.ceil(i * splitsize), Math.ceil((i + 1) * splitsize));
    }
    return chunks;
};

function extend(target) {
    var argsLen = arguments.length;
    var source = void 0;
    var keys = void 0;
    var key = void 0;
    var i = void 0;
    var j = void 0;

    for (j = 1; j < argsLen; j++) {
        source = arguments[j];
        keys = Object.keys(source);
        for (i = 0; i < keys.length; i++) {
            key = keys[i];
            target[key] = source[key];
        }
    }
    return target;
}

function makeRef(value) {
    return { value: value };
}

function setRef(ref) {
    ref.value = true;
}

function isSet(ref) {
    return !!ref.value;
}

function normalizeRangeSpec(spec) {
    return {
        from: spec.from,
        to: spec.to,
        fromInclusive: spec.hasOwnProperty('fromInclusive') ? spec.fromInclusive : true,
        toInclusive: spec.hasOwnProperty('toInclusive') ? spec.toInclusive : true
    };
}